###############################################################################
#
#    OERPScenario, OpenERP Functional Tests
#    Author Joel Grand-Guillaume 2009
#    Copyright Camptocamp SA
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 Afero of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


##############################################################################
#           Scenario: Validate a confirmed sale order
##############################################################################

##############################################################################
Given /^I have recorded on the (.*) a sale order of (.*) (.*) without tax called (\w+)$/ do |date,amount,curerncy_code,name|
  # Take first stockable product
  @product=ProductProduct.find(:first,:domain=>[['type','=','product']], :fields => ['id'])
  @product.should be_true
  # Take first customer partner with at least one address
  @partner=ResPartner.get_valid_partner({:type=>'customer',:fields => ['id']})
  @partner.should be_true
  currency_id = ResCurrency.find(:first, :domain=>[['code','=',curerncy_code]],:fields => ['id']).id
  currency_id.should be_true
  # Create an so with found product and given amount
  so = SaleOrder.new
  so.name = name
  so.date_order = Date.parse(str=date).to_s
  #auto-complete the address and other data based on the partner
  so.on_change('onchange_partner_id', :partner_id, @partner.id, @partner.id)
  so.pricelist_id=ProductPricelist.find(:first,:domain=>[['currency_id','=',currency_id]],:fields => ['id']).id
  so.order_line = [SaleOrderLine.new(:name => 'OERPScenario line', :product_id => @product.id, :price_unit => amount.to_f, :product_uom => 1)]
  so.create
  # Set it in the memorizer
  $utils.set_var(name.strip,so)
  $utils.get_var(name.strip).should be_true
  @saleorder=so

end

##############################################################################
When /^I press the confirm button$/ do
  # Call the 'invoice_open' method from account.invoice openobject
  @saleorder.wkf_action('order_confirm')
end

##############################################################################
Then /^I should see the sale order (\w+) open$/ do |name|
  @saleorder=$utils.get_var(name.strip)
  @saleorder.state.should == 'progress'
  # @saleorder.state.should == 'manual' or @saleorder.state.should == 'progress'
end

##############################################################################
Then /^the total amount = (.*)$/ do |amount|
  @saleorder.amount_total.should == amount.to_f
end


##############################################################################
#  Scenario: Validate exception when cancelling a related invoice
##############################################################################
Given /^change the shipping policy to 'Shipping & Manual Invoice'$/ do
  @saleorder.order_policy = 'manual'
  @saleorder.save
  @saleorder.order_policy.should == 'manual'
end

##############################################################################
Then /^I should see the sale order (\w+) manual in progress$/ do |name|
  @saleorder=$utils.get_var(name.strip)
  @saleorder.state.should == 'manual'
end

##############################################################################
When /^I press the create invoice button from SO$/ do
  @saleorder.wkf_action('manual_invoice')
end

##############################################################################
Then /^I should see the sale order (\w+) in progress$/ do |name|
  @saleorder=$utils.get_var(name.strip)
  @saleorder.state.should == 'progress'
end

##############################################################################
Then /^I should have (\w+) related draft invoice[s]? created$/ do |count|
  count = count.strip
  if count == 'a':
    count = 1
  end
  count = count.to_i
  @saleorder.invoice_ids.count.should == count
end

##############################################################################
Given /^I take the related (invoice|refund)$/ do |name|
  # @saleorder=SaleOrderLine.find(@saleorder.id)
  @invoice = false
  @saleorder.invoice_ids.each do |i|
    if i.type.split('_')[1] == name
      @invoice = i
    end
  end
  @invoice.should be_true
  # Set var to propagate the scope into invoice_step
end

##############################################################################
Given /^change the description for (\w+) and the date to (.*)$/ do |name,date|
  # require 'ruby-debug'
  # debugger
  date=Date.parse(str=date).to_s
  @invoice.name = name
  @invoice.date_invoice = date
  @invoice.save
  @invoice.name.should == name
  $utils.set_var(@invoice.name,@invoice)
  # @invoice.date.should == date
end

##############################################################################
When /^I press the cancel button on this invoice$/ do
  @invoice.wkf_action('invoice_cancel')
  @invoice.state.should == 'cancel'
end
##############################################################################
When /^then I press the set to draft button$/ do
  AccountInvoice.action_cancel_draft([@invoice.id])
  # @invoice.action_cancel_draft([@invoice.id])
  @invoice=AccountInvoice.find(@invoice.id)
  @invoice.save
end

##############################################################################
Then /^the SO should be in invoice exception$/ do
  @saleorder=SaleOrder.find(@saleorder.id)
  @saleorder.state.should == 'invoice_except'
end

##############################################################################
When /^then I press the invoice corrected button in the SO$/ do
  @saleorder.wkf_action('invoice_corrected')
end

##############################################################################
Then /^I should see the sale order MySimpleSO progress$/ do
  @saleorder.state.should == 'progress'
end

##############################################################################
#  Scenario: Create an invoice from a picking created from a SO
##############################################################################
Given /^change the shipping policy to 'Invoice from the Packing'$/ do
  @saleorder.order_policy = 'picking'
  @saleorder.save
  @saleorder.order_policy.should == 'picking'
end

##############################################################################
Given /^I take the related picking$/ do
  @picking = @saleorder.picking_ids[0]
  @picking.should be_true
end

##############################################################################
When /^I press the force assign button$/ do
  StockPicking.force_assign([@picking.id])
end

##############################################################################
Then /^the picking should be in (\w+)$/ do |state|
  @picking = StockPicking.find(@picking.id)
  state_names = {
    'available' => 'assigned',
    'done' => 'done',
  }
  @picking.state.should == state_names[state]
end

##############################################################################
Given /^I take the related stock moves$/ do
  @moves = @picking.move_lines
  @moves.should be_true
end

##############################################################################
Then /^the stock moves should be in (\w+)$/ do |state|
  state_names = {
    'available' => 'assigned',
    'done' => 'done',
  }
  @moves.each do |move| move.state.should == state_names[state] end
end

##############################################################################
When /^I call the (\w+) picking wizard$/ do |name|
  names = {
    'Partial' => 'stock.partial_picking',
    'Invoice' => 'stock.invoice_onshipping',
    'Return' => 'stock.return.picking',
  }
  @wizard = @picking.old_wizard_step(names[name])
  @wizard.should be_true
  @step_dict = {}
end

##############################################################################
When /^I confirm the whole (\w+) picking wizard$/ do |name|
  names = {
    'Partial' => 'move',
    'Return' => 'return',
  }
  lines = @wizard.open_object_resources[0].relations['move_lines']
  lines.each do |move_id|
    @step_dict["#{names[name]}#{move_id}"] =
      StockMove.read(move_id, ['product_qty'])["product_qty"]
  end
  if name == 'Partial'
    @wizard.split(@step_dict)
  else
    @wizard.return(@step_dict)
  end
end

##############################################################################
Then /^I should see the Invoicing wizard's selection as (\w+ \w+)$/ do |type|
  type_names = {
    'Customer Invoice' => 'out_invoice',
    'Supplier Invoice' => 'in_invoice',
    'Customer Refund' => 'out_refund',
    'Supplier Refund' => 'in_refund',
  }

  @wizard.datas[:type].should == type_names[type]
end

##############################################################################
And /^I set a (\w+) (\w+) journal in the wizard$/ do |refund, type|
  type = type.strip
  refund = refund.strip
  if refund == 'refund'
    refund = true
  else
    refund = false
  end
  journal = AccountJournal.find(:first, :domain => [['type','=',type],
    ['refund_journal','=',refund]], :fields => ['id', 'type', 'refund_journal'])
  journal.type.should == type
  journal.refund_journal.should == refund
  @step_dict = @wizard.datas.merge({:journal_id => journal.id, :group => false})
end

##############################################################################
Then /^I create the invoice with the wizard$/ do
  @wizard.create_invoice(@step_dict)
  @saleorder = SaleOrder.find(@saleorder.id)
end

##############################################################################
Then /^I should see the invoice type as (\w+ \w+)$/ do |type|
  type_names = {
    'Customer Invoice' => 'out_invoice',
    'Supplier Invoice' => 'in_invoice',
    'Customer Refund' => 'out_refund',
    'Supplier Refund' => 'in_refund',
  }
  @invoice.type.should == type_names[type]
end

##############################################################################
#  Scenario: The customer returns the goods and a refund invoice is created
##############################################################################
Given /^I take the sale order (\w+)$/ do |name|
  @saleorder = $utils.get_var(name.strip)
  @saleorder.should be_true
end

##############################################################################
And /^I set the wizard to be invoiced$/ do
  @step_dict = @wizard.datas.merge({:invoice_state => '2binvoiced'})
end

##############################################################################
Given /^I \w* take a related (\w+) type of picking$/ do |name|
  name = name.strip
  @saleorder = SaleOrder.find(@saleorder.id)
  @picking = false
  @saleorder.picking_ids.each do |p|
    if p.type == name
      @picking = p
    end
  end
  @picking.should be_true
end